<?php

namespace CleverReach\WooCommerce\Components\Repositories\Customers\HighPerformance;


use CleverReach\WooCommerce\Components\Repositories\Customers\Base_Receiver_Repository;

abstract class High_Performance_Base_Receiver_Repository extends Base_Receiver_Repository {

	/**
	 * Retrieves guest receivers by emails.
	 *
	 * @param array $emails List of emails.
	 *
	 * @return array
	 */
	protected function get_guest_receivers_by_emails( $emails ) {
		$prepared_emails = implode( "', '", esc_sql( $emails ) );
		$blog_prefix     = $this->get_db()->get_blog_prefix( get_current_blog_id() );

		$sql_query = "SELECT recipients.order_id as ID, recipients.date_created_gmt AS 'user_registered', recipients.email as user_email, recipients.first_name as _billing_first_name, recipients.last_name as _billing_last_name, recipients.address_1 as _billing_address_1, recipients.postcode as _billing_postcode, recipients.city as _billing_city, recipients.company as _billing_company, recipients.state as _billing_state, recipients.country as _billing_country, recipients.phone as _billing_phone
					  FROM (
         				SELECT orders.id as order_id, orders.date_created_gmt, order_meta.email, order_meta.first_name, order_meta.last_name, order_meta.address_1, order_meta.postcode, order_meta.city, order_meta.company, order_meta.state, order_meta.country, order_meta.phone,
                			ROW_NUMBER() OVER (PARTITION BY order_meta.email ORDER BY orders.id DESC) AS row_number
         				FROM {$this->get_db()->prefix}wc_orders orders JOIN {$this->get_db()->prefix}wc_order_addresses AS order_meta ON orders.id = order_meta.order_id
         				WHERE order_meta.address_type = 'billing'
           					AND order_meta.email IN ('$prepared_emails')
           					AND order_meta.email NOT IN (
             					SELECT u.user_email AS email
             					FROM {$this->get_db()->prefix}users u
                      			JOIN {$this->get_db()->prefix}usermeta capabilities ON u.ID = capabilities.user_id
             					WHERE capabilities.meta_key = '{$blog_prefix}capabilities'
               						AND u.user_email IN ('$prepared_emails')
         					)
           					AND orders.customer_id = '0'
     				  ) AS recipients
					  WHERE recipients.row_number = 1;";

		$receivers = $this->get_db()->get_results( $sql_query, ARRAY_A );

		if ( empty( $receivers ) ) {
			return array();
		}

		$receivers      = $this->sanitize( $receivers, static::GUEST_ID_PREFIX );
		$fetched_emails = $this->get_fetched_emails( $receivers, static::GUEST_ID_PREFIX );

		$orders_data = $this->get_order_repository()->get_order_data_for_guest_emails( $fetched_emails );

		return $this->merge_orders_data_with_receivers( $receivers, $orders_data, static::GUEST_ID_PREFIX );
	}

	/**
	 * @inheritDoc
	 * @param $receivers
	 * @param $user_prefix
	 *
	 * @return array
	 */
	protected function get_fetched_emails( $receivers, $user_prefix ) {
		$fetched_emails = array();
		foreach ( $receivers as $receiver ) {
			$fetched_emails[] = $receiver['user_email'];
		}

		return $fetched_emails;
	}

	/**
	 * @inheritDoc
	 * @param $array
	 * @param $user_prefix
	 *
	 * @return array
	 */
	protected function sanitize( $array, $user_prefix ) {
		$result = array();

		if ( self::CUSTOMER_ID_PREFIX === $user_prefix ) {
			return parent::sanitize( $array, $user_prefix );
		}

		foreach ( $array as $record ) {
			$index = $user_prefix . $record['ID'];

			$result[ $index ]                    = $record;
			$result[ $index ]['customer_number'] = $user_prefix . $record['ID'];
		}

		return $result;
	}
}