<?php

namespace CleverReach\WooCommerce\Components\Repositories;

use CleverReach\WooCommerce\Components\Util\Database;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ORM\Interfaces\ArchivedQueueItemRepository;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Exceptions\QueueItemSaveException;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\QueueItem;
use Exception;

class Archive_Repository extends Base_Repository implements ArchivedQueueItemRepository {

	/**
	 * Archive queue item
	 *
	 * @param QueueItem $queue_item
	 * @param array $additional_where
	 *
	 * @return int|void
	 * @throws QueueItemSaveException
	 */
	public function archiveQueueItem( QueueItem $queue_item, array $additional_where = array() ) {
		$this->table_name = $this->db->prefix . Database::ARCHIVE_TABLE;
		$item_id          = null;
		try {
			$item_id = $this->save_entity_to_storage( $queue_item );
		} catch ( Exception $exception ) {
			throw new QueueItemSaveException(
				'Failed to save queue item with id: ' . $item_id,
				0,
				$exception
			);
		}

		return $item_id;
	}

	/**
	 * @return string
	 */
	public static function getClassName() {
		return __CLASS__;
	}
}