<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Controllers;

use CleverReach\WooCommerce\Components\Services\BusinessLogic\Events\Group_Events_Service;
use CleverReach\WooCommerce\Components\Util\HTTP_Helper;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Group\GroupEventsService as Base_Group_Events_Service;
use WP_HTTP_Response;

/**
 * Class Clever_Reach_Group_Event_Webhook_Controller
 *
 * @package CleverReach\WooCommerce\Controllers
 */
class Clever_Reach_Group_Event_Webhook_Controller extends Clever_Reach_Base_Controller {

	/**
	 * Group events service.
	 *
	 * @var Group_Events_Service
	 */
	private $events_service;

	/**
	 * Is request call internal.
	 *
	 * @var bool
	 */
	protected $is_internal = false;

	/**
	 * Handles group event
	 */
	public function handle_group() {
		$response = new WP_HTTP_Response();
		$secret   = HTTP_Helper::get_param( 'secret' );
		if ( null === $secret ) {
			$response->set_status( 400 );
		} else {
			$token = $this->get_events_service()->getVerificationToken() . ' ' . $secret;
			$response->set_headers( array( 'Content-Type' => 'text/plain' ) );
			$response->set_data( $token );
			$response->set_status( 200 );
		}

		$this->return_plain_text( $response->get_data(), $response->get_status() );
	}

	private function get_events_service() {
		if ( null === $this->events_service ) {
			$this->events_service = ServiceRegister::getService( Base_Group_Events_Service::CLASS_NAME );
		}

		return $this->events_service;
	}
}
